# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

import xml.etree.cElementTree as ET

from driverng import MessageV2
from driverng import Worklist
from driverng import utils


class Driver(MessageV2):
    SPEC = {"code": "alere_reader", "label": "ALERE Reader"}

    def parse(self, data):
        return ET.fromstring(data.encode("utf8"))

    def to_string(self, message):
        return utils.prettify_xml(ET.tostring(message, "utf-8"))

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for svc in message.findall("SVC"):
            for observation_dttm in svc.findall("SVC.observation_dttm"):
                # Timezone is removed for compatibility purpose for Python 2.7 and 3.6
                date = datetime.datetime.strptime(
                    observation_dttm.get("V")[:19], "%Y-%m-%dT%H:%M:%S"
                )
            for opr in svc.findall("OPR"):
                for operator_id in opr.findall("OPR.operator_id"):
                    operator = operator_id.get("V")
            for rgt in svc.findall("RGT"):
                for name in rgt.findall("RGT.name"):
                    name = name.get("V")
                for lot_number in rgt.findall("RGT.lot_number"):
                    lot = lot_number.get("V")
                for expiration_date in rgt.findall("RGT.expiration_date"):
                    expiration = datetime.datetime.strptime(
                        expiration_date.get("V"), "%Y-%m-%d"
                    )
                # QC
                if message.tag == "OBS.R02":
                    for ctc in svc.findall("CTC"):
                        for name in ctc.findall("CTC.name"):
                            control_name = name.get("V")
                        for lot_number in ctc.findall("CTC.lot_number"):
                            control_lot = lot_number.get("V")
                        for level_cd in ctc.findall("CTC.level_cd"):
                            control_level = level_cd.get("V")
                        sample = worklist.get_or_create(
                            type="control",
                            sample_id=control_lot,
                            control_lot=control_lot,
                            control_level=control_level,
                            control_name=control_name,
                        )
                        for obs in ctc.findall("OBS"):
                            for observation_id in obs.findall("OBS.observation_id"):
                                code = observation_id.get("V")
                            for qualitative_value in obs.findall(
                                "OBS.qualitative_value"
                            ):
                                value = qualitative_value.get("V")
                            sample.results.add(code=code, value=value, date=date)
                # Patient
                if message.tag == "OBS.R01":
                    for pt in svc.findall("PT"):
                        for patient_id in pt.findall("PT.patient_id"):
                            sample_id = patient_id.get("V")
                        sample = worklist.get_or_create(
                            type="patient", sample_id=sample_id
                        )
                        for obs in pt.findall("OBS"):
                            for observation_id in obs.findall("OBS.observation_id"):
                                code = observation_id.get("V")
                            for qualitative_value in obs.findall(
                                "OBS.qualitative_value"
                            ):
                                value = qualitative_value.get("V")
                            result = sample.results.add(
                                code=code, value=value, date=date, operator=operator
                            )
                            result.lots.add(
                                type="reagent",
                                lot=lot,
                                name=name,
                                expiration=expiration,
                            )
        return worklist
